@@ -1,10 +1,12 @@
-import bioservices
 import urllib2
 from bs4 import BeautifulSoup
 import re
 import xml.etree.ElementTree as ET
 import pyproteinsExt.uniprot
 import json
+import os
+#from pkg_resources import resource_stream, Requirement
+#
 
 PSQ_FIELDS= ["idA", "idB", "altA", "altB", "aliasA", "aliasB", "interactionDetectionMethod", "firstAuthor", "pubid", "taxidA", "taxidB",
             "interactionTypes", "sourceDatabases", "interactionIdentifiers", "confidenceScore", "complexExpansion", "biologicalRoleA"
@@ -18,10 +20,6 @@ PSQ_FIELDS= ["idA", "idB", "altA", "altB", "aliasA", "aliasB", "interactionDetec
 class OLS():
 
     def __init__(self, ontology="mi"):
-        #self.cOntology = "MI"
-        #self.api = bioservices.WSDLService("OLS", "http://www.ebi.ac.uk/ontology-lookup/OntologyQuery.wsdl")
-        #self.api = bioservices.WSDLService("OLS", "http://www.ebi.ac.uk/ols/v2/OntologyQuery.wsdl")
-        #if ontology: self.cOntology = ontology
 
         self.cOntology = ontology
         self.restApi = 'http://www.ebi.ac.uk/ols/api/ontologies/' + self.cOntology + '/'
@@ -93,6 +91,14 @@ class PSICQUIC(object):
         self.records = []
         self.registredPublications = {}
 
+    def json(self, file=None):
+        jsonString = '{"type" : "mitabResult", "data" : [' + ','.join([ psqData.json for psqData in self ]) + '] }'
+
+        if file:
+            with open(file, 'w') as f:
+                f.write(jsonString)
+        return jsonString
+
     def dump(self, file=None):
         if file:
             with open(file, 'w') as f:
@@ -234,12 +240,11 @@ class PSICQUIC(object):
                 continue
             miql = self.registry[provider] + 'query/' + miqlString
             #print miql
-
-            ans = self._ping(miql + "?format=tab" + mitabLvl)
+            ans, encoder = self._ping(miql + "?format=tab" + mitabLvl)
             if ans == 0:
-                ans = self._ping(miql + "?format=tab25")
+                ans, encoder = self._ping(miql + "?format=tab25")
             if ans:
-                self._parse(ans)
+                self._parse(ans, encoder=encoder)
             else:
                 continue
 
@@ -262,8 +267,14 @@ class PSICQUIC(object):
             print url + "\n" + str(error.reason)
             return None
         raw = response.read()
+
+        encoder = response.headers['content-type'].split('charset=')[1] if len(response.headers['content-type'].split('charset=')) > 1 else 'utf-8'
+
+
+        #Content-Type:text/plain; charset=utf-8
+
         response.close()
-        return raw
+        return (raw, encoder)
 
     def getRegistry(self, url):
         try:
@@ -279,8 +290,19 @@ class PSICQUIC(object):
 
 
 
-    def _parse(self, raw):
-        bufferRecords = [PSQDATA(line) for line in raw.split("\n") if len(line) > 0]
+    def _parse(self, raw, **kwargs):
+
+        bufferRecords = []
+        for line in raw.split("\n"):
+            if len(line) == 0 or line.startswith("#"):
+                continue
+            if 'encoder' in kwargs:
+                #print "==>" + kwargs['encoder']
+                bufferRecords.append( PSQDATA( line.decode(kwargs['encoder']) ) )
+            else:
+                bufferRecords.append(PSQDATA(line))
+
+
         if self.mode is "LOOSE":
             self.records += bufferRecords
         else:
@@ -380,6 +402,29 @@ class PSICQUIC(object):
                     l += up
             return list(set(l))
 
+# Returns another instance of PSQ with filter data elements
+    def filter(self, **kwargs):
+        target = PSICQUIC()
+        buf = kwargs['uniprot']
+        # For now we dont look into alias uniprot identifiers
+        if 'uniprot' in kwargs:
+            if isinstance(kwargs['uniprot'], list):
+                buf = set(kwargs['uniprot'])
+            elif isinstance(kwargs['uniprot'], basestring):
+                buf = set(kwargs['uniprot'])
+#            elif isinstance(kwargs['uniprot'], set):
+
+            for psqData in self:
+                up = psqData['uniprotPair']
+                if not up:
+                    continue
+               # print up
+               # print buf
+                if set(up) & buf :
+                    #print str(up) + ' <===> ' + str(buf)
+                    target.records.append(psqData)
+
+        return target
 
 # This is the main interface we try to define smart __getitem__() access keys
 class PSQDATA():
@@ -393,6 +438,10 @@ class PSQDATA():
     def __init__(self, raw):
         self.raw = raw
         self.data = [PSQDATUM(column) for column in raw.split("\t")]
+
+        if len(self.data) != 15 and len(self.data) != 42:
+            raise ValueError ("Uncorrect numbner of tabulated fields on input " + str(len(data)) )
+
     def __repr__(self):
         string = "\t".join(map(str,self.data))
         return string
@@ -408,6 +457,8 @@ class PSQDATA():
             return self.data[12].data[0].value
         if key is "interactionDetectionMethod":
             return self.data[6].data[0].value
+        if key is "species":
+            return (self.data[9].data[0].value, self.data[10].data[0].value)
         if key is 'uniprotPair':
             a = pyproteinsExt.uniprot.capture(self.data[0].data[0].value) if pyproteinsExt.uniprot.capture(self.data[0].data[0].value) else pyproteinsExt.uniprot.capture(self.data[2].data[0].value)
             b = pyproteinsExt.uniprot.capture(self.data[1].data[0].value) if pyproteinsExt.uniprot.capture(self.data[1].data[0].value) else pyproteinsExt.uniprot.capture(self.data[3].data[0].value)
@@ -416,9 +467,25 @@ class PSQDATA():
                 return (a, b)
             return None
 
+    @property
+    def json(self):
+
+        #for k,d in zip(PSQ_FIELDS, self.data):
+
+            #print k
+            #print d
+            #print d.data
+            #print map(str,d.data)
+
+            #print [ x.replace('"', r'\\') for x in map(str,d.data)]
+
+        return '{' + ','.join([ '"' + k  + '" : [' + ','.join([ '"' + x.replace('"', r'\"') + '"' for x in map(str,d.data) ]) + ']' for k,d in zip(PSQ_FIELDS, self.data) ]) + '}'
+
     @property
     def interactors(self):
-        return  (self.data[0].content + self.data[2].content, self.data[1].content + self.data[3].content )
+        datum = (self.data[0].content + self.data[2].content, self.data[1].content + self.data[3].content )
+        #print datum
+        return  datum
 
     def hasInteractors(self, mode='STRICT'):
     #    for psqDatum in [ self.data[0].content + self.data[]
@@ -473,40 +540,34 @@ class PSQFIELD():
 
     def __str__(self):
 
+
         try :
             string = self.value.decode('ascii')
         except UnicodeDecodeError:
-            print "OUPS"
-
+        #    print "Decode error" + self.value
+            string = self.value.decode('utf8')
         except UnicodeEncodeError:
             #print "UTF ENCODING"
             string = self.value.encode('utf8')
-            #print "OUPS 22 >>" + string
-            #return '-'
-        #else:
-        #    print "String PSQ field error"
-        #    print self.value
-        #    string = '-'
+        #string = self.value
 
-        #string = unicode(string)
-
-        if self.type: string = self.type + string
+        if self.type:
+            string = self.type + string
         if self.annotation:
-            #print 'There are annotations'
-            #print type(string)
-            #print type(self.annotation)
+
             try :
                 str_annot = self.annotation.decode('ascii')
             except UnicodeDecodeError:
-                print "OUPS ANNNOT"
+                #print "OUPS ANNNOT"
+                str_annot = self.annotation.decode('utf8')
             except UnicodeEncodeError:
-                print "OUPSS EEECC"
+                #print "OUPSS EEECC"
                 str_annot = self.annotation.encode('utf8')
-
-            string = string + "(" + str(str_annot) + ")"
+        #    string = string + "(" + str(str_annot) + ")"
+            string = string + "(" + str(self.annotation) + ")"
         else:
             pass
-        #    print 'There are no annotation'
+
         return string
 
 
@@ -524,7 +585,19 @@ class registry():
     def __init__(self, raw):
         if raw:
             #print raw
-            root = ET.fromstring(raw)
+            try:
+                root = ET.fromstring(raw)
+            except:
+                print "Registry Remote XML parsing error, using local file as registry"
+#                resource_stream(Requirement.parse("pyproteinsExt=="), "restez/httpconn.py")
+
+                path =  os.path.abspath(__file__)
+                dir_path = os.path.dirname(path)
+                tree = ET.parse(dir_path + '/static/psicquicRegistryDefault.xml')
+                root = tree.getroot()
+            #else:
+
+
             for child in root:
                 name = ""
                 url = ""
